//  7-segment display bit patterns 
//  for each digital value: 0 to F
//
//     AAAA        0000
//    F    B      5    1
//    F    B      5    1
//    F    B      5    1
//     GGGG        6666
//    E    C      4    2
//    E    C      4    2
//    E    C      4    2
//     DDDD  H     3333  7
//


//  LOGISIM
#macro _a  (1 << 5)
#macro _b  (1 << 4)
#macro _c  (1 << 1)
#macro _d  (1 << 2)
#macro _e  (1 << 3)
#macro _f  (1 << 6)
#macro _g  (1 << 7)
#macro _dp (1 << 0)
segments = [ 7, 2, 3, 4, 1, 0, 5, 6 ];



patterns = [( _a | _b | _c | _d | _e | _f |  0),    //  0
            (  0 | _b | _c |  0 |  0 |  0 |  0),    //  1
            ( _a | _b |  0 | _d | _e |  0 | _g),    //  2
            ( _a | _b | _c | _d |  0 |  0 | _g),    //  3
            (  0 | _b | _c |  0 |  0 | _f | _g),    //  4
            ( _a |  0 | _c | _d |  0 | _f | _g),    //  5
            ( _a |  0 | _c | _d | _e | _f | _g),    //  6
            ( _a | _b | _c |  0 |  0 |  0 |  0),    //  7
            ( _a | _b | _c | _d | _e | _f | _g),    //  8
            ( _a | _b | _c | _d |  0 | _f | _g),    //  9
            ( _a | _b | _c |  0 | _e | _f | _g),    //  A
            (  0 |  0 | _c | _d | _e | _f | _g),    //  B
            ( _a |  0 |  0 | _d | _e | _f |  0),    //  C
            (  0 | _b | _c | _d | _e |  0 | _g),    //  D
            ( _a |  0 |  0 | _d | _e | _f | _g),    //  E
            ( _a |  0 |  0 |  0 | _e | _f | _g)];   //  F
            
_minus = _g;                                        //  -
_hex   = _c | _e | _f | _g;                         //  h
_octal = _c | _d | _e | _g;                         //  o


//  7-segment decoder ROM table for values 0-255,
//  one byte for each mode, digit, and value.
table = array_create(8 * 4 * 256);

for (var value=0; value<256; value++) {
    
    //  UNSIGNED mode
    var v = value;
    for (var digit=0; digit<4; digit++) {
        if (max(1,v) >= power(10,digit)) {
            var p = patterns[floor(v / power(10,digit)) mod 10];
        } else {
            var p = 0;
        }
        table[1024*0 + 256*digit + value] = p;
    }
    
    //  SIGNED mode
    if (value > 127) {
        var v = (0xFF ^ value)+1;
    } else {
        var v = value;
    }
    for (var digit=0; digit<4; digit++) {
        if (max(1,v) >= power(10,digit)) {
            var p = patterns[floor(v / power(10,digit)) mod 10];
        } else if (value > 127 && digit == 3) {
            var p = _minus;
        } else {
            var p = 0;
        }
        table[1024*1 + 256*digit + value] = p;
    }
    
    //  HEXADECIMAL mode
    var v = value;
    for (var digit=0; digit<4; digit++) {
        if (digit == 3) {
            var p = _hex;
        } else if (digit == 2) {
            var p = 0;
        } else {
            var p = patterns[floor(v / power(16,digit)) mod 16];
        }
        table[1024*2 + 256*digit + value] = p;
    }
    
    //  OCTAL mode
    var v = value;
    for (var digit=0; digit<4; digit++) {
        if (digit == 3) {
            var p = _octal;
        } else {
            var p = patterns[floor(v / power(8,digit)) mod 8];
        }
        table[1024*3 + 256*digit + value] = p;
    }
    
    //  ANIMATION mode
    var v = value;
    var p;
    for (var digit=0; digit<4; digit++) {
        switch (v mod 12) {
            case 0: 
                p = select(digit, 0, 0, _a, _a);
                break;
            case 1:
                p = select(digit, 0, _a, _a, 0);
                break;
            case 2:
                p = select(digit, _a, _a, 0, 0);
                break;
            case 3:
                p = select(digit, _a|_b, 0, 0, 0);
                break;
            case 4:
                p = select(digit, _b|_c, 0, 0, 0);
                break;
            case 5:
                p = select(digit, _c|_d, 0, 0, 0);
                break;
            case 6:
                p = select(digit, _d, _d, 0, 0);
                break;
            case 7:
                p = select(digit, 0, _d, _d, 0);
                break;
            case 8:
                p = select(digit, 0, 0, _d, _d);
                break;
            case 9:
                p = select(digit, 0, 0, 0, _d|_e);
                break;
            case 10:
                p = select(digit, 0, 0, 0, _e|_f);
                break;
            case 11:
                p = select(digit, 0, 0, 0, _a|_f);
                break;
        }
        table[1024*4 + 256*digit + value] = p;
    }

}